(function (Drupal, drupalSettings, once) {
  Drupal.behaviors.salesforceMFWSetup = {
    attach: (context) => {
      if (!once('salesforceMFWSetup', 'html').length) {
        return;
      }

      const salesforceConfig = JSON.parse(drupalSettings.salesforce_mfw);

      /**
       * Initializes with the embeddedservice_bootstrap of the Salesforce utility
       * to provide preChat field setup before loading the utility instance.
       */
      function initEmbeddedMessaging() {
        try {
          const embeddedserviceBootstrap = embeddedservice_bootstrap || {};

          embeddedserviceBootstrap.settings.language =
            salesforceConfig.language;

          window.addEventListener('onEmbeddedMessagingReady', () => {
            const prechatHiddenFields = {};
            const prechatVisibleFields = {};
            // Iterate over the fields and translate into expected format.
            Object.values(salesforceConfig.fields).forEach((field) => {
              if (field.hidden) {
                prechatHiddenFields[field.name] = field.default_value;
              } else {
                prechatVisibleFields[field.name] = {
                  value: field.default_value,
                  isEditableByEndUser: Boolean(field.editable_by_user),
                };
              }
            });

            // Set visible pre-chat fields
            embeddedserviceBootstrap.prechatAPI.setVisiblePrechatFields(
              prechatVisibleFields,
            );

            // Set hidden pre-chat fields
            embeddedserviceBootstrap.prechatAPI.setHiddenPrechatFields(
              prechatHiddenFields,
            );
          });

          embeddedserviceBootstrap.init(
            salesforceConfig.organization_id,
            salesforceConfig.config_name,
            salesforceConfig.site_url,
            { scrt2URL: salesforceConfig.snippet_config },
          );
        } catch (err) {
          console.error('Error loading Embedded Messaging: ', err);
        }
      }

      // Dispatch an event so that third-parties are able to override configuration
      // via Javascript event listeners.
      window.dispatchEvent(
        new CustomEvent('SalesforceMFWConfig', {
          detail: salesforceConfig,
        }),
      );

      if (salesforceConfig.hasOwnProperty('allowed_paths')) {
        let matched = false;
        const storageKey = `${salesforceConfig.organization_id}_WEB_STORAGE`;
        const salesforceJWT =
          typeof Storage !== 'undefined'
            ? JSON.parse(localStorage.getItem(storageKey))
            : false;

        matched =
          salesforceJWT &&
          'JWT' in salesforceJWT &&
          salesforceConfig.allow_path_override;

        // Check to see if we're already matched due to a chat session being active.
        if (!matched) {
          const paths = salesforceConfig.allowed_paths.split(/\r\n|\n|\r/);
          const currentPath = window.location.pathname;
          for (let i = 0; i < paths.length; i++) {
            const pattern = `^${paths[i].replace(/\*/g, '.*')}$`;
            const regex = new RegExp(pattern);
            if (regex.test(currentPath)) {
              matched = true;
              break;
            }
          }
        }

        if (matched) {
          window.onload = function () {
            const script = document.createElement('script');
            script.type = 'text/javascript';
            script.src = salesforceConfig.utility_bootstrap;
            script.onload = initEmbeddedMessaging;
            document.head.appendChild(script);
          };
        }
      }
    },
  };
})(Drupal, drupalSettings, once);
